package geniusweb.domaineditor.model.profile;

import java.math.BigDecimal;

import geniusweb.domaineditor.model.NumberValueSetModel;
import geniusweb.profile.utilityspace.NumberValueSetUtilities;
import geniusweb.profile.utilityspace.ValueSetUtilities;
import tudelft.utilities.listener.DefaultListenable;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.mvc.model.NumberModel;
import tudelft.utilities.mvc.model.RestrictedNumberModel;
import tudelft.utilities.mvc.model.events.Event;
import tudelft.utilities.mvc.panels.PopupReporter;

/**
 * Contains info for a {@link NumberValueSetUtilities}
 *
 */
public class NumberValueSetUtilitiesModel extends DefaultListenable<Event>
		implements ValueSetUtilitiesModel {

	private static final BigDecimal DEFAULT = new BigDecimal("0.5");
	// this reporter in theory is never used.
	// it's connected to values that never change
	private final static Reporter dummylog = new PopupReporter(null);
	private final static NumberModel LOW = new NumberModel(BigDecimal.ZERO,
			dummylog);
	private final static NumberModel HIGH = new NumberModel(BigDecimal.ONE,
			dummylog);
	private final NumberModel lowValue;
	private final RestrictedNumberModel lowUtility;
	private final NumberModel highValue;
	private final RestrictedNumberModel highUtility;

	/**
	 * 
	 * @param lowValue
	 * @param lowUtility  a {@link RestrictedNumberModel} with min=0, max=100
	 * @param highValue
	 * @param highUtility a {@link RestrictedNumberModel} with min=0, max=100
	 */
	NumberValueSetUtilitiesModel(NumberModel lowValue,
			RestrictedNumberModel lowUtility, NumberModel highValue,
			RestrictedNumberModel highUtility) {
		this.lowValue = lowValue;
		this.lowUtility = lowUtility;
		this.highValue = highValue;
		this.highUtility = highUtility;
	}

	NumberValueSetUtilitiesModel(NumberValueSetModel valuesmodel,
			Reporter log) {
		this(valuesmodel.getLow(),
				new RestrictedNumberModel(DEFAULT, LOW, HIGH, log),
				valuesmodel.getHigh(),
				new RestrictedNumberModel(DEFAULT, LOW, HIGH, log));
	}

	public NumberModel getLowValue() {
		return lowValue;
	}

	public RestrictedNumberModel getLowUtility() {
		return lowUtility;
	}

	public NumberModel getHighValue() {
		return highValue;
	}

	public RestrictedNumberModel getHighUtility() {
		return highUtility;
	}

	@Override
	public NumberValueSetUtilities getCurrentValue()
			throws IllegalStateException {
		return new NumberValueSetUtilities(lowValue.getValue(),
				lowUtility.getValue(), highValue.getValue(),
				highUtility.getValue());
	}

	@Override
	public void setCurrentValue(ValueSetUtilities obj)
			throws IllegalArgumentException {
		if (!(obj instanceof NumberValueSetUtilities))
			throw new IllegalArgumentException(
					"Expected instance of NumberValueSetUtilities but got "
							+ obj);
		NumberValueSetUtilities utils = (NumberValueSetUtilities) obj;
		// FIXME this may fail if new lowvalue > old highvalue
		lowValue.setValue(utils.getLowValue());
		lowUtility.setValue(utils.getLowUtility());
		highValue.setValue(utils.getHighValue());
		highUtility.setValue(utils.getHighUtility());
	}
};